#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

# file      : build_pot_file.py
# project   : xbpf
# copyright : (C) 2014 by Claude BULIN
#
# This file is part of xbpf project
#
# xbpf is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; version 3.
#
# xbpf is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA


import os, sys, subprocess, gettext, datetime

from fconfig import FConfig
import Language_Init
from prefs import Preferences

try:
	import polib
except:
	if sys.stdout.isatty() == True:
		print( "")
		print( "xbpf (Build Pot File) ")
		print( ">>>  " + _("Please install python-polib module") + "  <<<" )
		print( "")
	exit(0)

import urllib
import urllib.request
import http.client
import socket
from datetime import date


class build_pot_file():

	def __init__( self, Form=None, langage_folder_path="", ListParse=None, ListPotFile=None, Bool_CompletionTranslate=False, Bool_ForceTranslate=False ):
		self.langage_folder_path      = langage_folder_path
		self.path_po                  = os.path.join(langage_folder_path, 'po')
		self.path_po_pot              = os.path.join(langage_folder_path, 'po/pot')
		self.ListParse                = ListParse
		self.Form                     = Form
		self.ListPotFile              = ListPotFile
		self.Bool_CompletionTranslate = Bool_CompletionTranslate
		self.Bool_ForceTranslate      = Bool_ForceTranslate
		self.list_syntax_error        = []

	#
	# Read files and product file all.pot
	#
	def make_xgettext( self ):
		tab_scan = []
		counter_pot = 1

		print( "Read directories:" )

		for list_parse in self.ListParse :

			tab_scan[:] = []
			for root, dirs, files in os.walk( self.langage_folder_path ):
				if(files):
					for f in files:
						fileName, fileExtension = os.path.splitext( f )
						if list_parse == fileExtension:
							if not root in tab_scan:
								tab_scan.append( root )

			if tab_scan :
				print( " "*4, list_parse.upper()," : ", len(tab_scan) )
				for tab in tab_scan:
					os.chdir( tab )
					command = ""
					command = command + "xgettext --keyword=_ --sort-output -o "
					command = command + "'" + self.path_po_pot + "/ALL_" + str(counter_pot) + ".pot' *" + list_parse
					print( " "*8, tab,":" )
					print( " "*8, command )
					subprocess.call(command, shell=True)
					counter_pot += 1

		print( "Use sed from" + " [ " + self.path_po_pot + " ]" )
		os.chdir( self.path_po_pot )
		command = ""
		command = command + "sed -i 's/charset=CHARSET/charset=UTF-8/g' *.pot"
		print( " "*4, command )
		subprocess.call(command, shell=True)


	#
	# Add datas files.pot to all.pot file
	# msgcat be applied to all files.pot
	#
	def make_msgcat(self):
		os.chdir( self.path_po_pot )
		command = ""
		for root, dirs, files in os.walk(self.path_po_pot):
			if files:
				for f in files:
					command = command + "./" + f + " "
		if command != "":
			command = command + "-F -o " + "'" + self.path_po + "/ALL.pot'"
			command = "msgcat " + command
			print( " "*4, command )
			subprocess.call(command, shell=True)
		else:
			print( " "*4,"No file *.pot" )


	#
	# Creating file(s).po if not exist
	#
	def create_langues(self):
		# header of POT file
		header_text = ""
		header_text = header_text + '# NAME_PROJECT POT Template File.\n'
		header_text = header_text + '# Copyright (C) %s  YOUR_NAME\n' % date.today().year
		header_text = header_text + '# This file is distributed under the same license as the PACKAGE package.\n'
		header_text = header_text + '# YOUR_NAME <ADR_MAIL>, %s.\n' % date.today().year
		header_text = header_text + '#\n'
		header_text = header_text + 'msgid ""\n'
		header_text = header_text + 'msgstr ""\n'
		header_text = header_text + '"Project-Id-Version: NAME_PROJECT (version: XXX)\\n"\n'
		header_text = header_text + '"Report-Msgid-Bugs-To: YOUR_NAME <ADR_MAIL>\\n"\n'
		po_date = datetime.datetime.now()
		header_text = header_text + '"POT-Creation-Date: %s\\n"\n' % po_date
		header_text = header_text + '"PO-Revision-Date: %s\\n"\n' % po_date
		header_text = header_text + '"Last-Translator: YOUR_NAME <ADR_MAIL>\\n"\n'
		header_text = header_text + '"Language-Team: URL\\n"\n'
		header_text = header_text + '"MIME-Version: 1.0\\n"\n'
		header_text = header_text + '"Content-Type: text/plain; charset=UTF-8\\n"\n'
		header_text = header_text + '"Content-Transfer-Encoding: 8bit\\n"\n'
		os.chdir( self.path_po )
		dirs = os.listdir( self.path_po )
		bool_create = False
		for LN in self.ListPotFile:
			if not LN in dirs:
				print( " "*4, "Create", LN )
				f = open(LN, "w")
				f.write(header_text)
				f.close()
				bool_create = True
		if bool_create == False:
			print( " "*4, "The files exist" )


	#
	# Add datas all.pot to the files.p
	#
	def make_msgmerge(self):
		os.chdir(self. path_po )
		dirs = os.listdir( self.path_po )
		for file in dirs:
			if os.path.isfile(file):
				fileName, fileExtension = os.path.splitext( file )
				if ".po" == fileExtension:
					command = "msgmerge --quiet " + file + " ALL.pot -U"
					print( " "*4, command )
					subprocess.call(command, shell=True)


	#
	# Product file(s).mo from file(s).po
	#
	def make_msgfmt(self):
		bool_error = False
		os.chdir( self.path_po )
		dirs = os.listdir( self.path_po )
		for file in dirs:
			if os.path.isfile(file):
				fileName, fileExtension = os.path.splitext( file )
				if ".po" == fileExtension:
					command = "msgfmt "+ file + " --output-file=" + fileName + ".mo"
					print( " "*4, command )
					if subprocess.call(command, shell=True) != 0:
						bool_error = True
		return bool_error


	#
	# From
	#	http://stackoverflow.com/questions/17440343/python-checking-internet-connection-more-than-once
	# Recoded
	#
	def test_internet_connection(self):
		BoolConnected = False
		test_con_url = "www.google.com" # For connection testing
		test_con_resouce = "/intl/en/policies/privacy/" # may change in future
		test_con = http.client.HTTPConnection(test_con_url) # create a connection

		try:
			test_con.request("GET", test_con_resouce) # do a GET request
			response = test_con.getresponse()
		except httplib.ResponseNotReady as e:
			# print( "Improper connection state" )
			BoolConnected = False
		except socket.gaierror as e:
			# print( "Not connected" )
			BoolConnected = False
		else:
			# print( "Connected" )
			BoolConnected = True

		test_con.close()

		return BoolConnected


	#
	# From
	#	https://pypi.python.org/pypi/pytranslate/0.1.3
	#	https://github.com/stockrt/pytranslate
	# Recoded
	#
	def get_translating(self, text, sl='auto', tl='portuguese'):

		BoolErrorNetwork = False;

		urllib.request.FancyURLopener.version = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19'
		f = urllib.request.FancyURLopener()
		post_params = urllib.parse.urlencode({'client':'t', 'text':text, 'sl':'%s' % sl.lower(), 'tl':'%s' % tl.lower()})
		try:
			content = f.open('http://translate.google.com/translate_a/t', post_params).read()
			Buffer = content.decode()
			begin = Buffer.find( '"' )
			begin += 1
			end   = Buffer.find( '"', begin )
			StringTranslate = ""
			while begin < end:
				StringTranslate += Buffer[ begin ]
				begin += 1
			BoolErrorNetwork = False;
		except:
			StringTranslate = ""
			BoolErrorNetwork = True;

		return BoolErrorNetwork, text, StringTranslate


	#
	# From
	#	https://bitbucket.org/izi/polib/wiki/Home
	# Recoded
	#
	def translate( self, pathnamefile, BoolForce = False ):

		BoolErrorNetwork = False

		if self.test_internet_connection() == False:
			print( " "*4, ">>>  " + _("No network connection") + "  <<<")
			self.list_syntax_error.append( _("No network connection") )
			return

		# Get name file without ext
		translate_to = os.path.splitext(os.path.basename(pathnamefile))[0]
		# The first two characters
		translate_to = translate_to[0:2]
		# Open file 'po'
		try:
			po = polib.pofile( pathnamefile )
			# Browse file 'po'
			for entry in po:
				# Do not force the translation
				if BoolForce == False:
					if entry.msgstr == "":
						# Delete the carriage returns
						String = entry.msgid.replace( '\n', ' ' )
						# Translate
						BoolErrorNetwork, str_origine, str_trans = self.get_translating( String, sl='auto', tl=translate_to )
						# Assignment of translation
						entry.msgstr = str_trans
				# Force translation
				else:
					# Delete the carriage returns
					String = entry.msgid.replace( '\n', ' ' )
					# Translate
					BoolErrorNetwork, str_origine, str_trans = self.get_translating( String, sl='auto', tl=translate_to )
					# Assignment of translation
					entry.msgstr = str_trans
				if BoolErrorNetwork == True:
					print( " "*4, ">>>  " + _("No network connection from google") + "  <<<")
					self.list_syntax_error.append( _("No network connection from google") )
					return self.list_syntax_error

			# Save 'po' file with translation
			po.save()
		except:
			print("ERREUR DANS LE FICHIER: ", pathnamefile )
			self.list_syntax_error.append( pathnamefile )

		for err in self.list_syntax_error:
			print("-------------------------------- %s ", err )
		return


	#
	# create_pot_file
	#
	def create_pot_file(self):
		bool_syntax_error = False
		list_syntax_error = []

		if not os.path.isdir( self.path_po_pot ):
			os.makedirs( self.path_po_pot )

		print("\nRead files [ *.py, *.glade ] and product file all.pot")
		print("-------------------------------------------------------")
		self.make_xgettext()

		print("\nAdd datas files.pot to all.pot file\nmsgcat be applied to all files.pot")
		print("-------------------------------------------------------")
		self.make_msgcat()

		print("\nCreating file(s).po if not exist")
		print("-------------------------------------------------------")
		self.create_langues()

		print("\nAdd datas all.pot to the files.po")
		print("-------------------------------------------------------")
		self.make_msgmerge()

		if self.Bool_CompletionTranslate == True or self.Bool_ForceTranslate == True:
			print(  _("Translating files From the network ...") )
			os.chdir( self.path_po )
			dirs = os.listdir( self.path_po )
			for file in dirs:
				if os.path.isfile(file):
					fileName, fileExtension = os.path.splitext( file )
					if ".po" == fileExtension:
						print( " "*4, file )
						if self.Bool_CompletionTranslate == True:
							self.translate( file, BoolForce = False )
						elif self.Bool_ForceTranslate == True:
							self.translate( file, BoolForce = True )

		print("\nProduct file(s).mo from file(s).po")
		print("-------------------------------------------------------")
		bool_syntax_error = self.make_msgfmt()

		print("\nRemove directory po/pot/")
		print("-------------------------------------------------------")
		os.chdir( self.path_po )
		command = "rm -rf pot/"
		subprocess.call(command, shell=True)

		print("\nRemove files: po/*~")
		print("-------------------------------------------------------")
		print("\n")
		os.chdir( self.path_po )
		command = "rm -f *~"
		subprocess.call(command, shell=True)

		return self.list_syntax_error, bool_syntax_error


